"""RoboDock Actions - Core functionality for data and model management."""

import boto3
import requests
import logging
import threading
import concurrent.futures
from botocore.config import Config
from typing import Any, Optional, Callable
import zipfile
import tarfile
import os
import time
from pathlib import Path


def safe_extract_zip(zip_path: str, extract_dir: str, strip_top_level: bool = True) -> None:
    """安全解压ZIP文件，防止zip-slip攻击。
    
    Args:
        zip_path: ZIP文件路径
        extract_dir: 解压目标目录
        strip_top_level: 是否移除顶层目录
    """
    extract_dir_new = Path(extract_dir).resolve()
    extract_dir_new.mkdir(parents=True, exist_ok=True)

    with zipfile.ZipFile(zip_path, 'r') as z:
        # 列出所有文件名（排除目录条目）
        names = [n for n in z.namelist() if n and not n.endswith('/')]
        # 检测是否存在统一的顶层目录名
        prefix = None
        if strip_top_level and names:
            first_parts = [Path(n).parts[0] for n in names if Path(n).parts]
            if first_parts and all(p == first_parts[0] for p in first_parts):
                prefix = first_parts[0] + '/'

        for member in z.infolist():
            name = member.filename
            if prefix and name.startswith(prefix):
                rel = name[len(prefix):]
            else:
                rel = name
            if not rel:
                continue  # 跳过根目录条目
            target_path = (extract_dir_new / rel).resolve()

            # 安全检查：防 zip-slip（确保目标路径在 extract_dir 下）
            if not str(target_path).startswith(str(extract_dir_new) + os.sep):
                raise Exception(f"Unsafe zip member path: {name}")

            target_path.parent.mkdir(parents=True, exist_ok=True)
            if member.is_dir():
                continue
            with z.open(member) as src, open(target_path, "wb") as dst:
                dst.write(src.read())


class RoboDockActions:
    """RoboDock核心操作类，提供数据上传下载、任务管理等功能。"""

    _upload_executor: Optional[concurrent.futures.ThreadPoolExecutor] = None
    _upload_executor_lock = threading.Lock()
    _upload_pending_slots: Optional[threading.BoundedSemaphore] = None

    def __init__(
        self,
        s3_endpoint: Optional[str] = None,
        s3_access_key: Optional[str] = None,
        s3_secret_key: Optional[str] = None,
        api_url: Optional[str] = None,
        bucket_name: Optional[str] = None,
        region_name: Optional[str] = None,
    ) -> None:
        """初始化RoboDock Actions客户端。
        
        Args:
            s3_endpoint: S3存储服务端点
            s3_access_key: S3访问密钥
            s3_secret_key: S3密钥
            api_url: RoboDock API URL
            bucket_name: S3存储桶名称
            region_name: S3区域名称
        """
        # 优先使用入参；未传入时回退到环境变量；环境变量未设置则用兜底值
        s3_endpoint = s3_endpoint or os.getenv("S3_ENDPOINT_URL", "http://192.168.1.6:9000")
        s3_access_key = s3_access_key or os.getenv("AWS_ACCESS_KEY_ID", "robodock")
        s3_secret_key = s3_secret_key or os.getenv("AWS_SECRET_ACCESS_KEY", "robodock")
        api_url = api_url or os.getenv("ROBOCK_SERVER_URL", "http://robodock-service:80")
        bucket_name = bucket_name or os.getenv("S3_BUCKET_NAME", "robodock")
        region_name = region_name or os.getenv("S3_REGION_NAME", "auto")

        self.s3_client = boto3.client(
            "s3",
            endpoint_url=s3_endpoint,
            aws_access_key_id=s3_access_key,
            aws_secret_access_key=s3_secret_key,
            config=Config(
                connect_timeout=10,
                read_timeout=600,
                retries={"max_attempts": 3, "mode": "standard"},
                signature_version="s3v4",
            ),  # MinIO 推荐使用 v4 签名
            region_name=region_name,
        )
        self.bucket_name = bucket_name
        self.api_singledata_prefix = "api/p/v1/singledata"
        self.api_run_prefix = "api/p/v1/run"
        self.api_dataset_prefix = "api/p/v1/dataset"
        self.server_url = api_url
        
        # 确保bucket存在
        try:
            self.s3_client.head_bucket(Bucket=self.bucket_name)
        except:
            self.s3_client.create_bucket(Bucket=self.bucket_name)

    def _default_upload_success(self, s3_key: str, *, outdir_path: str, upload_path: str) -> None:
        logging.info(
            "Background upload success: local=%s upload_path=%s s3_key=%s",
            outdir_path,
            upload_path,
            s3_key,
        )

    def _default_upload_error(self, exc: Exception, *, outdir_path: str, upload_path: str) -> None:
        logging.exception(
            "Background upload failed: local=%s upload_path=%s err=%s",
            outdir_path,
            upload_path,
            exc,
        )

    @classmethod
    def _get_upload_executor(cls) -> concurrent.futures.ThreadPoolExecutor:
        """Global upload executor with bounded pending queue.

        - `UPLOAD_MAX_WORKERS`: 并发上传线程数
        - `UPLOAD_MAX_PENDING`: 允许排队的任务数（包含正在执行的）
        """
        with cls._upload_executor_lock:
            if cls._upload_executor is None:
                max_workers = int(os.getenv("UPLOAD_MAX_WORKERS", "4"))
                max_pending = int(os.getenv("UPLOAD_MAX_PENDING", "8"))
                max_workers = max(1, max_workers)
                max_pending = max(max_workers, max_pending)

                cls._upload_executor = concurrent.futures.ThreadPoolExecutor(
                    max_workers=max_workers,
                    thread_name_prefix="robodock-upload",
                )
                cls._upload_pending_slots = threading.BoundedSemaphore(value=max_pending)
            return cls._upload_executor

    @classmethod
    def wait_for_uploads(cls, timeout: Optional[float] = None) -> None:
        """Wait for all queued upload tasks.

        Note: `timeout` applies to executor shutdown; if tasks hang beyond S3 timeouts,
        this can still block.
        """
        with cls._upload_executor_lock:
            executor = cls._upload_executor
            cls._upload_executor = None
            cls._upload_pending_slots = None
        if executor is not None:
            executor.shutdown(wait=True)

    def get_dataset_info(self, uuid: str) -> dict:
        """获取数据集信息。
        
        Args:
            uuid: 数据集UUID
            
        Returns:
            数据集信息字典
            
        Raises:
            Exception: 获取数据集信息失败
        """
        response = requests.get(f"{self.server_url}/{self.api_dataset_prefix}?uuid={uuid}")
        if response.status_code != 200:
            raise Exception(f"Failed to get dataset info for {uuid}: {response.text}")
        return response.json()

    def get_singledata_info(self, uuid: str) -> dict:
        """获取单条数据信息。
        
        Args:
            uuid: 数据UUID
            
        Returns:
            数据信息字典
            
        Raises:
            Exception: 获取数据信息失败
        """
        response = requests.get(f"{self.server_url}/{self.api_singledata_prefix}/{uuid}")
        if response.status_code != 200:
            raise Exception(f"Failed to get singledata info for {uuid}: {response.text}")
        return response.json()

    def get_run_info(self, unique_name: str, unique_type: str = "uuid") -> dict:
        """获取运行任务信息。
        
        Args:
            unique_name: 任务唯一标识
            unique_type: 标识类型，可选"uuid"或"unique_name"
            
        Returns:
            任务信息字典
            
        Raises:
            Exception: 获取任务信息失败
        """
        response = requests.get(
            f"{self.server_url}/{self.api_run_prefix}/{unique_name}?unique_type={unique_type}"
        )
        if response.status_code != 200:
            raise Exception(f"Failed to get run info for {unique_name}: {response.text}")
        return response.json()

    def download_data(self, data_name: str, uuid: str, dirname: str) -> str:
        """从S3下载并解压数据。
        
        Args:
            data_name: 数据文件名
            uuid: 数据UUID
            dirname: S3上的目录名
            
        Returns:
            解压后的本地目录路径
            
        Raises:
            Exception: 不支持的文件类型或下载失败
        """
        tmp_dir = Path.cwd() / "tmp"
        tmp_dir.mkdir(parents=True, exist_ok=True)
        download_path = str(tmp_dir / f"{uuid}_{data_name}")
        extract_path = str(tmp_dir / uuid)

        self.s3_client.download_file(self.bucket_name, f"{dirname}/{data_name}", download_path)
        logging.info(f"Downloaded {data_name} to {download_path}")

        # 根据文件扩展名解压
        if data_name.endswith('.zip'):
            safe_extract_zip(download_path, extract_path)
            logging.info(f"Extracted zip to {extract_path}")
        elif data_name.endswith('.tar.gz') or data_name.endswith('.tgz'):
            with tarfile.open(download_path, 'r:gz') as tar:
                tar.extractall(path=extract_path)
            logging.info(f"Extracted tar.gz to {extract_path}")
            os.remove(download_path)
        elif data_name.endswith('.tar'):
            with tarfile.open(download_path, 'r') as tar:
                tar.extractall(path=extract_path)
            logging.info(f"Extracted tar to {extract_path}")
            os.remove(download_path)
        else:
            raise Exception(
                f"Unsupported file type: {data_name}. Supported: .zip, .tar, .tar.gz, .tgz"
            )

        return extract_path

    def download_checkpoint(self, online_path: str, local_path: str) -> str:
        """从S3下载检查点文件并解压。
        
        Args:
            online_path: S3上的文件路径
            local_path: 本地解压目录
            
        Returns:
            解压后的路径
            
        Raises:
            Exception: 不支持的文件类型
        """
        filename = online_path.split('/')[-1]
        download_path = f"{local_path}/{filename}"

        # 确保本地目录存在
        Path(local_path).mkdir(parents=True, exist_ok=True)

        self.s3_client.download_file(self.bucket_name, online_path, download_path)
        logging.info(f"Downloaded {filename} to {download_path}")

        # 解压文件
        if filename.endswith('.zip'):
            extract_path = download_path.rstrip('.zip')
            safe_extract_zip(download_path, extract_path)
            logging.info(f"Extracted zip to {extract_path}")
            return extract_path
        elif filename.endswith('.tar.gz') or filename.endswith('.tgz'):
            with tarfile.open(download_path, 'r:gz') as tar:
                tar.extractall(path=local_path)
            logging.info(f"Extracted tar.gz to {local_path}")
            os.remove(download_path)
            return local_path
        elif filename.endswith('.tar'):
            with tarfile.open(download_path, 'r') as tar:
                tar.extractall(path=local_path)
            logging.info(f"Extracted tar to {local_path}")
            os.remove(download_path)
            return local_path
        else:
            raise Exception(f"Unsupported file type: {filename}")

    def upload_data(self, outdir_path: str, upload_path: str) -> str:
        """压缩本地目录并上传到S3。
        
        Args:
            outdir_path: 本地输出目录路径
            upload_path: S3上传路径
            
        Returns:
            S3对象键
            
        Raises:
            FileNotFoundError: 输出目录不存在
            Exception: 压缩或上传失败
        """
        outdir = Path(outdir_path)
        if not outdir.exists():
            raise FileNotFoundError(f"Output directory does not exist: {outdir}")

        tar_name = f"{outdir.name}.tar"
        tmp_tar = outdir.parent / tar_name
        s3_key = f"{upload_path.rstrip('/')}/{tar_name}".lstrip('/')

        try:
            logging.info("Creating tar %s from %s", tmp_tar, outdir)
            with tarfile.open(tmp_tar, "w") as tar:
                tar.add(outdir, arcname=outdir.name)

            logging.info("Uploading %s to s3://%s/%s", tmp_tar, self.bucket_name, s3_key)
            self.s3_client.upload_file(str(tmp_tar), self.bucket_name, s3_key)
            logging.info("Upload successful: s3://%s/%s", self.bucket_name, s3_key)
        except Exception as e:
            logging.error("Failed to create or upload tar: %s", e)
            raise
        finally:
            # 清理临时tar文件
            try:
                if tmp_tar.exists():
                    tmp_tar.unlink()
                    logging.debug("Removed temporary tar %s", tmp_tar)
            except Exception as cleanup_err:
                logging.warning("Failed to remove temporary tar %s: %s", tmp_tar, cleanup_err)
        return s3_key

    def upload_data_async(
        self,
        outdir_path: str,
        upload_path: str,
        on_success: Optional[Callable[[str], None]] = None,
        on_error: Optional[Callable[[Exception], None]] = None,
    ) -> Optional[concurrent.futures.Future[None]]:
        """异步上传数据，队列满时返回None。

        Args:
            outdir_path: 本地输出目录路径
            upload_path: S3上传路径
            on_success: 成功回调（参数为s3_key）
            on_error: 失败回调（参数为异常）

        Returns:
            Future对象或None（队列已满）
        """
        if on_success is None:
            on_success = lambda s3_key: self._default_upload_success(
                s3_key, outdir_path=outdir_path, upload_path=upload_path
            )

        if on_error is None:
            on_error = lambda exc: self._default_upload_error(
                exc, outdir_path=outdir_path, upload_path=upload_path
            )

        executor = self._get_upload_executor()
        slots = self.__class__._upload_pending_slots
        if slots is None:
            slots = threading.BoundedSemaphore(value=1)
            self.__class__._upload_pending_slots = slots

        if not slots.acquire(blocking=False):
            logging.warning(
                "Upload queue full; skip upload: local=%s upload_path=%s",
                outdir_path,
                upload_path,
            )
            return None

        def _run() -> None:
            try:
                s3_key = self.upload_data(outdir_path=outdir_path, upload_path=upload_path)
                on_success(s3_key)
            except Exception as exc:
                on_error(exc)
            finally:
                try:
                    slots.release()
                except Exception:
                    pass

        return executor.submit(_run)

    def upload_checkpoint_async(
        self,
        *,
        outdir_path: str,
        upload_path: str,
        run_unique_name: str,
        unique_type: str = "unique_name",
        run_field: str = "check_point_path",
    ) -> Optional[concurrent.futures.Future[None]]:
        """异步上传checkpoint目录，并在成功后更新run。"""

        def _on_success(s3_key: str) -> None:
            self._default_upload_success(s3_key, outdir_path=outdir_path, upload_path=upload_path)
            self.update_run(run_unique_name, {run_field: s3_key}, unique_type=unique_type)

        def _on_error(exc: Exception) -> None:
            self._default_upload_error(exc, outdir_path=outdir_path, upload_path=upload_path)
            self.update_run(
                run_unique_name,
                {
                    "status": "failed",
                    "finish_time": int(time.time() * 1000),
                    "fail_info": str(exc),
                },
                unique_type=unique_type,
            )
            os._exit(1)

        return self.upload_data_async(
            outdir_path=outdir_path,
            upload_path=upload_path,
            on_success=_on_success,
            on_error=_on_error,
        )

    def update_singledata(self, uuid: str, checker_result: str, checker_info: str) -> int:
        """更新单条数据的检查结果。
        
        Args:
            uuid: 数据UUID
            checker_result: 检查结果
            checker_info: 检查信息
            
        Returns:
            HTTP状态码
            
        Raises:
            Exception: 更新失败
        """
        data = {
            "any_in_dataformat": {
                "checker_info": checker_info,
                "checker_result": checker_result
            }
        }
        response = requests.patch(
            f"{self.server_url}/{self.api_singledata_prefix}/{uuid}",
            json=data
        )
        logging.info(f"Update result: {response.status_code}, {response.text}")
        if response.status_code != 200:
            logging.error(f"Failed to update singledata for {uuid}: {response.text}")
            raise Exception(f"Failed to update singledata for {uuid}: {response.text}")
        return response.status_code

    def update_run(self, unique_name: str, body: dict, unique_type: str = "uuid") -> int:
        """更新运行任务状态。
        
        Args:
            unique_name: 任务唯一标识
            body: 更新内容字典
            unique_type: 标识类型
            
        Returns:
            HTTP状态码
            
        Raises:
            Exception: 更新失败
        """
        logging.info(f"Updating run {unique_name} with body: {body}")
        response = requests.patch(
            f"{self.server_url}/{self.api_run_prefix}/{unique_name}?unique_type={unique_type}",
            json=body
        )
        logging.info(f"Update result: {response.status_code}, {response.text}")
        if response.status_code != 200:
            logging.error(f"Failed to update run for {unique_name}: {response.text}")
            raise Exception(f"Failed to update run for {unique_name}: {response.text}")
        return response.status_code



"""测试工具函数."""

import pytest
import os
from unittest.mock import patch
from robodock_sdk.utils import (
    get_env_config,
    is_resume_enabled,
    get_unique_name
)


def test_get_env_config_default():
    """测试默认配置."""
    with patch.dict(os.environ, {}, clear=True):
        config = get_env_config()
        
        assert config['PROJECT_NAME'] == 'default_project'
        assert config['RUN_NAME'] == 'default_run'
        assert config['BATCH_SIZE'] == '32'


def test_get_env_config_custom():
    """测试自定义配置."""
    env_vars = {
        'PROJECT_NAME': 'my_project',
        'RUN_NAME': 'run_001',
        'BATCH_SIZE': '64'
    }
    
    with patch.dict(os.environ, env_vars, clear=True):
        config = get_env_config()
        
        assert config['PROJECT_NAME'] == 'my_project'
        assert config['RUN_NAME'] == 'run_001'
        assert config['BATCH_SIZE'] == '64'


def test_is_resume_enabled_true():
    """测试恢复训练启用."""
    with patch.dict(os.environ, {'RESUME': 'true'}, clear=True):
        assert is_resume_enabled() is True
    
    with patch.dict(os.environ, {'RESUME': 'True'}, clear=True):
        assert is_resume_enabled() is True


def test_is_resume_enabled_false():
    """测试恢复训练禁用."""
    with patch.dict(os.environ, {'RESUME': 'false'}, clear=True):
        assert is_resume_enabled() is False
    
    with patch.dict(os.environ, {}, clear=True):
        assert is_resume_enabled() is False


def test_get_unique_name():
    """测试生成唯一名称."""
    # 使用参数
    name = get_unique_name('project1', 'run1')
    assert name == 'project1-run1'
    
    # 使用环境变量
    with patch.dict(os.environ, {'PROJECT_NAME': 'proj', 'RUN_NAME': 'run'}, clear=True):
        name = get_unique_name()
        assert name == 'proj-run'



"""RoboDock SDK utilities."""

import os
from typing import Dict, Optional


def get_env_config() -> Dict[str, str]:
    """从环境变量获取配置信息。
    
    Returns:
        配置字典
    """
    return {
        'PROJECT_NAME': os.getenv('PROJECT_NAME', 'default_project'),
        'RUN_NAME': os.getenv('RUN_NAME', 'default_run'),
        'SAVE_INTERVAL': os.getenv('SAVE_INTERVAL', '5000'),
        'DATASET_DIR': os.getenv('DATASET_DIR', ''),
        'BATCH_SIZE': os.getenv('BATCH_SIZE', '32'),
        'NUM_TRAIN_STEPS': os.getenv('NUM_TRAIN_STEPS', '20000'),
        'LOG_INTERVAL': os.getenv('LOG_INTERVAL', '10'),
        'WANDB_ENABLED': os.getenv('WANDB_ENABLED', 'false'),
        'WANDB_API_KEY': os.getenv('WANDB_API_KEY', ''),
        'PRETRAIN_MODEL': os.getenv('PRETRAIN_MODEL', ''),
        'RESUME': os.getenv('RESUME', 'false'),
        'RESUME_FROM': os.getenv('RESUME_FROM', ''),
        'OUTPUT_DIR': os.getenv('OUTPUT_DIR', '/workspace/checkpoints'),
        'DATASET_UUID': os.getenv('DATASET_UUID', ''),
    }


def is_resume_enabled() -> bool:
    """检查是否启用恢复训练。
    
    Returns:
        是否启用恢复
    """
    return os.getenv('RESUME', 'false').lower() == 'true'


def get_unique_name(project_name: Optional[str] = None, run_name: Optional[str] = None) -> str:
    """生成唯一的运行名称。
    
    Args:
        project_name: 项目名称
        run_name: 运行名称
        
    Returns:
        唯一名称
    """
    project = project_name or os.getenv('PROJECT_NAME', 'default_project')
    run = run_name or os.getenv('RUN_NAME', 'default_run')
    return f"{project}-{run}"



"""RoboDock SDK - Python SDK for RoboDock Platform."""

__version__ = "0.1.0"

from .client import RoboDockClient
from .actions import RoboDockActions

__all__ = ["RoboDockClient", "RoboDockActions", "__version__"]



"""RoboDock Client - High-level interface for RoboDock SDK."""

from typing import Optional
from .actions import RoboDockActions


class RoboDockClient:
    """RoboDock客户端，提供便捷的高层API。"""

    @staticmethod
    def _validate_non_empty(name: str, value: Optional[str]) -> None:
        if value is None:
            return
        if not isinstance(value, str) or not value.strip():
            raise ValueError(f"{name} must be a non-empty string when provided")

    def __init__(
        self,
        s3_endpoint: Optional[str] = None,
        s3_access_key: Optional[str] = None,
        s3_secret_key: Optional[str] = None,
        api_url: Optional[str] = None,
        bucket_name: Optional[str] = None,
        region_name: Optional[str] = None
    ):
        """初始化RoboDock客户端。
        
        Args:
            s3_endpoint: S3存储服务端点
            s3_access_key: S3访问密钥
            s3_secret_key: S3密钥
            api_url: RoboDock API URL
            bucket_name: S3存储桶名称
            region_name: S3区域名称
        """
        self._validate_non_empty("s3_endpoint", s3_endpoint)
        self._validate_non_empty("s3_access_key", s3_access_key)
        self._validate_non_empty("s3_secret_key", s3_secret_key)
        self._validate_non_empty("api_url", api_url)
        self._validate_non_empty("bucket_name", bucket_name)
        self._validate_non_empty("region_name", region_name)

        kwargs = {}
        if s3_endpoint:
            kwargs['s3_endpoint'] = s3_endpoint
        if s3_access_key:
            kwargs['s3_access_key'] = s3_access_key
        if s3_secret_key:
            kwargs['s3_secret_key'] = s3_secret_key
        if api_url:
            kwargs['api_url'] = api_url
        if bucket_name:
            kwargs['bucket_name'] = bucket_name
        if region_name:
            kwargs['region_name'] = region_name
            
        self.actions = RoboDockActions(**kwargs)

    def get_dataset(self, uuid: str):
        """获取数据集信息。
        
        Args:
            uuid: 数据集UUID
            
        Returns:
            数据集信息
        """
        return self.actions.get_dataset_info(uuid)

    def get_data(self, uuid: str):
        """获取单条数据信息。
        
        Args:
            uuid: 数据UUID
            
        Returns:
            数据信息
        """
        return self.actions.get_singledata_info(uuid)

    def get_run(self, unique_name: str, unique_type: str = "uuid"):
        """获取运行任务信息。
        
        Args:
            unique_name: 任务唯一标识
            unique_type: 标识类型
            
        Returns:
            任务信息
        """
        return self.actions.get_run_info(unique_name, unique_type)

    def download_dataset(self, data_name: str, uuid: str, dirname: str = "singledata"):
        """下载数据集。
        
        Args:
            data_name: 数据文件名
            uuid: 数据UUID
            dirname: S3目录名
            
        Returns:
            解压后的本地路径
        """
        return self.actions.download_data(data_name, uuid, dirname)

    def download_checkpoint(self, online_path: str, local_path: str):
        """下载检查点。
        
        Args:
            online_path: S3路径
            local_path: 本地路径
            
        Returns:
            解压后的路径
        """
        return self.actions.download_checkpoint(online_path, local_path)

    def upload_model(self, outdir_path: str, upload_path: str):
        """上传模型。
        
        Args:
            outdir_path: 本地输出目录
            upload_path: S3上传路径
            
        Returns:
            S3对象键
        """
        return self.actions.upload_data(outdir_path, upload_path)

    def upload_model_async(self, outdir_path: str, upload_path: str):
        """异步上传模型。

        Args:
            outdir_path: 本地输出目录
            upload_path: S3上传路径

        Returns:
            Future对象或None（队列已满）
        """
        return self.actions.upload_data_async(outdir_path, upload_path)

    def upload_checkpoint_async(
        self,
        *,
        outdir_path: str,
        upload_path: str,
        run_unique_name: str,
        unique_type: str = "unique_name",
        run_field: str = "check_point_path",
    ):
        """异步上传checkpoint并更新run。

        Args:
            outdir_path: 本地checkpoint目录
            upload_path: S3上传路径
            run_unique_name: 运行唯一标识
            unique_type: 标识类型
            run_field: 写回run的字段名

        Returns:
            Future对象或None（队列已满）
        """
        return self.actions.upload_checkpoint_async(
            outdir_path=outdir_path,
            upload_path=upload_path,
            run_unique_name=run_unique_name,
            unique_type=unique_type,
            run_field=run_field,
        )

    def update_run_status(self, unique_name: str, status: str, **kwargs):
        """更新任务状态。
        
        Args:
            unique_name: 任务唯一标识
            status: 状态（processing/success/failed）
            **kwargs: 其他更新字段（start_time, finish_time, fail_info等）
            
        Returns:
            HTTP状态码
        """
        body = {"status": status, **kwargs}
        return self.actions.update_run(unique_name, body, unique_type="unique_name")

    def update_singledata(self, uuid: str, checker_result: str, checker_info: str):
        """更新单条数据的检查结果。
        
        Args:
            uuid: 数据UUID
            checker_result: 检查结果
            checker_info: 检查信息
            
        Returns:
            HTTP状态码
        """
        return self.actions.update_singledata(uuid, checker_result, checker_info)



"""测试RoboDockActions类."""

import pytest
import threading
from unittest.mock import Mock, patch, MagicMock
from pathlib import Path
from robodock_sdk.actions import RoboDockActions


@pytest.fixture
def actions():
    """创建RoboDockActions实例用于测试."""
    RoboDockActions.wait_for_uploads()
    with patch.dict(
        "os.environ",
        {
            "S3_BUCKET_NAME": "robodock",
            "S3_REGION_NAME": "us-east-1",
        },
        clear=False,
    ):
        with patch('boto3.client'):
            instance = RoboDockActions(
                s3_endpoint="http://test:9000",
                s3_access_key="test",
                s3_secret_key="test",
                api_url="http://test:8080"
            )
            yield instance
    RoboDockActions.wait_for_uploads()


def test_init(actions):
    """测试初始化."""
    assert actions.server_url == "http://test:8080"
    assert actions.bucket_name == "robodock"


def test_init_uses_env_fallbacks():
    """未传入参数时应使用环境变量回退值."""
    with patch.dict(
        "os.environ",
        {
            "S3_ENDPOINT_URL": "http://env:9000",
            "AWS_ACCESS_KEY_ID": "env-ak",
            "AWS_SECRET_ACCESS_KEY": "env-sk",
            "ROBOCK_SERVER_URL": "http://env:8080",
            "S3_BUCKET_NAME": "env-bucket",
            "S3_REGION_NAME": "env-region",
        },
    ):
        with patch("boto3.client") as mock_client:
            RoboDockActions.wait_for_uploads()
            actions = RoboDockActions()
            mock_client.assert_called_once()
            _, kwargs = mock_client.call_args
            assert kwargs["endpoint_url"] == "http://env:9000"
            assert kwargs["aws_access_key_id"] == "env-ak"
            assert kwargs["aws_secret_access_key"] == "env-sk"
            assert kwargs["region_name"] == "env-region"
            assert actions.server_url == "http://env:8080"
            assert actions.bucket_name == "env-bucket"


def test_get_dataset_info_success(actions):
    """测试获取数据集信息成功."""
    with patch('requests.get') as mock_get:
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {"data": {"uuid": "test-uuid"}}
        mock_get.return_value = mock_response
        
        result = actions.get_dataset_info("test-uuid")
        
        assert result == {"data": {"uuid": "test-uuid"}}
        mock_get.assert_called_once()


def test_get_dataset_info_failure(actions):
    """测试获取数据集信息失败."""
    with patch('requests.get') as mock_get:
        mock_response = Mock()
        mock_response.status_code = 404
        mock_response.text = "Not found"
        mock_get.return_value = mock_response
        
        with pytest.raises(Exception) as exc_info:
            actions.get_dataset_info("test-uuid")
        
        assert "Failed to get dataset info" in str(exc_info.value)


def test_update_run_success(actions):
    """测试更新运行状态成功."""
    with patch('requests.patch') as mock_patch:
        mock_response = Mock()
        mock_response.status_code = 200
        mock_patch.return_value = mock_response
        
        result = actions.update_run("test-run", {"status": "success"})
        
        assert result == 200
        mock_patch.assert_called_once()


def test_update_run_failure(actions):
    """测试更新运行状态失败."""
    with patch('requests.patch') as mock_patch:
        mock_response = Mock()
        mock_response.status_code = 400
        mock_response.text = "Bad request"
        mock_patch.return_value = mock_response
        
        with pytest.raises(Exception) as exc_info:
            actions.update_run("test-run", {"status": "success"})
        
        assert "Failed to update run" in str(exc_info.value)


def test_upload_data_async_executes(actions):
    """测试异步上传会执行并返回Future."""
    done_event = threading.Event()

    def _fake_upload(outdir_path, upload_path):
        done_event.set()
        return "s3://bucket/key"

    with patch.dict("os.environ", {"UPLOAD_MAX_WORKERS": "1", "UPLOAD_MAX_PENDING": "1"}):
        with patch.object(actions, "upload_data", side_effect=_fake_upload) as mock_upload:
            future = actions.upload_data_async("/tmp/out", "models/test")
            assert future is not None
            future.result(timeout=2)
            assert done_event.is_set()
            mock_upload.assert_called_once()


def test_upload_data_async_queue_full(actions):
    """测试异步上传队列满时返回None."""
    block_event = threading.Event()

    def _blocking_upload(outdir_path, upload_path):
        block_event.wait(timeout=2)
        return "s3://bucket/key"

    with patch.dict("os.environ", {"UPLOAD_MAX_WORKERS": "1", "UPLOAD_MAX_PENDING": "1"}):
        with patch.object(actions, "upload_data", side_effect=_blocking_upload):
            first = actions.upload_data_async("/tmp/out1", "models/test1")
            assert first is not None
            second = actions.upload_data_async("/tmp/out2", "models/test2")
            assert second is None
            block_event.set()
            first.result(timeout=2)


def test_wait_for_uploads_clears_executor(actions):
    """测试等待异步上传会清理执行器."""
    done_event = threading.Event()

    def _fake_upload(outdir_path, upload_path):
        done_event.set()
        return "s3://bucket/key"

    with patch.dict("os.environ", {"UPLOAD_MAX_WORKERS": "1", "UPLOAD_MAX_PENDING": "1"}):
        with patch.object(actions, "upload_data", side_effect=_fake_upload):
            future = actions.upload_data_async("/tmp/out", "models/test")
            assert future is not None
            RoboDockActions.wait_for_uploads()
            assert done_event.is_set()
            assert RoboDockActions._upload_executor is None
            assert RoboDockActions._upload_pending_slots is None



"""测试RoboDockClient类."""

import pytest
from unittest.mock import Mock, patch
from robodock_sdk import RoboDockClient


@pytest.fixture
def client():
    """创建RoboDockClient实例用于测试."""
    with patch('boto3.client'):
        return RoboDockClient(
            s3_endpoint="http://test:9000",
            api_url="http://test:8080"
        )


def test_client_init(client):
    """测试客户端初始化."""
    assert client.actions is not None
    assert client.actions.server_url == "http://test:8080"


def test_client_init_rejects_empty_string():
    """传入空字符串应抛出异常."""
    with patch("boto3.client"):
        with pytest.raises(ValueError):
            RoboDockClient(s3_endpoint=" ")


def test_get_dataset(client):
    """测试获取数据集."""
    with patch.object(client.actions, 'get_dataset_info') as mock_get:
        mock_get.return_value = {"data": {"uuid": "test"}}
        
        result = client.get_dataset("test-uuid")
        
        assert result == {"data": {"uuid": "test"}}
        mock_get.assert_called_once_with("test-uuid")


def test_update_run_status(client):
    """测试更新运行状态."""
    with patch.object(client.actions, 'update_run') as mock_update:
        mock_update.return_value = 200
        
        result = client.update_run_status(
            unique_name="test-run",
            status="success",
            finish_time=123456
        )
        
        assert result == 200
        mock_update.assert_called_once()
        call_args = mock_update.call_args
        assert call_args[0][0] == "test-run"
        assert call_args[0][1]["status"] == "success"
        assert call_args[0][1]["finish_time"] == 123456



"""Setup script for RoboDock SDK."""

from setuptools import setup, find_packages
from pathlib import Path

# Read the contents of README file
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text(encoding='utf-8')

setup(
    name="robodock-sdk",
    version="0.1.0",
    author="kingcansky",
    author_email="1811077651@163.com",
    description="Python SDK for RoboDock Platform - Data and Model Management",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/FII-AILab/robodock-sdk.git",
    packages=find_packages(exclude=["tests", "examples", "docs"]),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    python_requires=">=3.8",
    install_requires=[
        "boto3>=1.26.0",
        "requests>=2.28.0",
        "botocore>=1.29.0",
    ],
    extras_require={
        "dev": [
            "pytest>=7.0.0",
            "pytest-cov>=4.0.0",
            "black>=22.0.0",
            "flake8>=5.0.0",
            "mypy>=0.990",
        ],
    },
    entry_points={
        "console_scripts": [
            "robodock-train=robodock_action:main",
        ],
    },
)
